#!/usr/share/venvs/netq-apps/bin/python
#
# Copyright (c) 2017-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import sys
import os
import json
import datetime
import requests
import requests.exceptions
from tabulate import tabulate
from netq_lib.common.utils import RED_T, COLOR_GREEN
from netq_lib.common.utils import subprocess_wrapper, is_cloud_opta, is_bcm

opta_service = "netq-opta"
endpoint_gw_service = "netq-endpoint-gateway"
exclude_list = ["netq-lcm-executor"]

opta_service_cmd = ["kubectl", "get", "services", "netq-opta-api", "-o", "json"]

def get_service_ip(is_cloud=False):
    if not is_cloud:
        opta_service_cmd[3] = endpoint_gw_service

    if is_bcm():
        opta_service_cmd.extend(['-n', 'netq'])

    try:
        out = subprocess_wrapper(opta_service_cmd)
    except Exception:
        print('Failed to run command {}'.format(opta_service_cmd))
        sys.exit(1)

    json_out = json.loads(out)

    return json_out['spec']['clusterIP']

def get_opta_info(is_cloud=False):
    opta_info_json = ""
    service_ip = get_service_ip(is_cloud)
    service_port = 8080
    print("Service IP: ", service_ip)

    if (is_cloud):
        url = 'http://{}:{}/health/agent-info'.format(service_ip, service_port)
    else:
        url = 'http://{}:{}/netq/control/v1/health/agent-info'.format(service_ip, service_port)

    try:
        response = requests.get(url=url, headers={'accept': 'application/json'})
        if response.status_code != requests.codes.ok:
            raise requests.exceptions.HTTPError(response.json())

        opta_info_json = response.json()
        #print(opta_info_json)
        return opta_info_json
    except Exception as exc:
        print('GET request failed for url {}'.format(url))
        raise requests.exceptions.HTTPError("Request error") from exc

    return opta_info_json

def dump_opta_cloud_health_info(opta_info_json):
    headers = ['Opta Health Status', 'Opta-Gateway Channel Status']
    info = opta_info_json.get('healthData')
    if info.get('isHealthy') is True:
        health_status = COLOR_GREEN('Healthy')
    else:
        health_status = RED_T('Unhealthy')

    gw_status = RED_T('Unknown')
    meta_data= info.get('metaData')
    for entry in meta_data:
        if entry.get('name') == 'gateway_channel_state':
            gw_status_str = entry.get('value')
            if gw_status_str == 'READY':
                gw_status = COLOR_GREEN(gw_status_str)
            else:
                gw_status = RED_T(gw_status_str)
            break
    opta_health = [health_status, gw_status]
    print((tabulate([opta_health], headers=headers, tablefmt='simple')))

def dump_opta_agent_info(opta_info_json, is_cloud=False):
    table = []
    headers = ['Agent ID', 'Remote Address', 'Status', 'Messages Exchanged', 'Time Since Last Communicated']

    if (is_cloud):
        info = opta_info_json.get('agentsInfo')
    else:
        info = opta_info_json.get('info')

    agent_info_list = info.get('agentsInfoList')
    for agent_info in agent_info_list:
        exclude = any(map(agent_info.get('aid').__contains__, exclude_list))
        if exclude == True:
            continue

        if agent_info.get('status') == 'UP':
            status = COLOR_GREEN(agent_info.get('status'))
        else:
            status = RED_T(agent_info.get('status'))

        last_message_time = datetime.datetime.fromtimestamp(int(agent_info.get('lastMessageTime')) / 1000 )

        table.append([agent_info.get('aid'), agent_info.get('remoteAddress'), status, agent_info.get('messageCount'), last_message_time])

    print((tabulate(table, headers=headers, tablefmt='simple')))

def dump_opta_info(opta_info_json, is_cloud=False, json_out=False):
    if json_out:
        print(opta_info_json)

    if is_cloud:
        dump_opta_cloud_health_info(opta_info_json)
        print("\n")

    dump_opta_agent_info(opta_info_json, is_cloud)

if __name__ == '__main__':
    prog_name = os.path.basename(sys.argv[0])
    if os.getuid():
        print('Must run {} as root'.format(prog_name))
        sys.exit(1)

    is_cloud = is_cloud_opta()

    opta_info_json = ""
    try:
        opta_info_json = get_opta_info(is_cloud)
    except Exception:
        print('Failed retrieving opta information')
        sys.exit(1)

    if len(opta_info_json) > 0:
        dump_opta_info(opta_info_json, is_cloud=is_cloud, json_out=False)
